@@ -38,22 +38,29 @@ readFirst :: String             -- ^ filename
 
 readFirst name demand path warn =
     case name of
-       '/':nm -> try nm   [""]
-       _      -> try name (cons dd (".":path))
+                       -- Windows drive in absolute path
+       c:':':'\\':nm-> try nm   (Just (c:':':[])) [""]
+       c:':':'/':nm -> try nm   (Just (c:':':[])) [""]
+                       -- Windows drive in relative path
+       c:':':nm     -> try nm   (Just (c:':':[])) (cons dd (".":path))
+                       -- unix-like absolute path
+       '/':nm       -> try nm   Nothing           [""]
+                       -- any relative path
+       _            -> try name Nothing           (cons dd (".":path))
   where
     dd = directory demand
     cons x xs = if null x then xs else x:xs
-    try name [] = do
+    try name _ [] = do
         when warn $
           hPutStrLn stderr ("Warning: Can't find file \""++name
                            ++"\" in directories\n\t"
                            ++concat (intersperse "\n\t" (cons dd (".":path)))
                            ++"\n  Asked for by: "++show demand)
         return ("missing file: "++name,"")
-    try name (p:ps) = do
-        let file = cleanPath p++'/':cleanPath name
+    try name drive (p:ps) = do
+        let file = (maybe id (++) drive) $ cleanPath p++'/':cleanPath name
         ok <- doesFileExist file
-        if not ok then try name ps
+        if not ok then try name drive ps
           else do content <- readFileUTF8 file
                   return (file,content)
 
